# Ref: https://github.com/DavidAnson/markdownlint/blob/main/doc/Rules.md
all

# First header should be a top level header
exclude_rule 'MD002'
# Unordered list style
exclude_rule 'MD004'
# Unordered list indentation
exclude_rule 'MD007'
# Trailing spaces
exclude_rule 'MD009'
# Multiple consecutive blank lines
exclude_rule 'MD012'
# Line length
exclude_rule 'MD013'
# No space after hash on atx style header
exclude_rule 'MD018'
# Headers should be surrounded by blank lines
exclude_rule 'MD022'
# Headers must start at the beginning of the line
exclude_rule 'MD023'
# Multiple headers with the same content
exclude_rule 'MD024'
# Multiple top level headers in the same document
exclude_rule 'MD025'
# Trailing punctuation in header
exclude_rule 'MD026'
# Ordered list item prefix
exclude_rule 'MD029'
# Fenced code blocks should be surrounded by blank lines
exclude_rule 'MD031'
# Lists should be surrounded by blank lines
exclude_rule 'MD032'
# Inline HTML
exclude_rule 'MD033'
# Bare URLs
exclude_rule 'MD034'
# Spaces inside emphasis markers
exclude_rule 'MD037'
# Fenced code blocks should have a language specified
exclude_rule 'MD040'
# First line in file should be a top level header
exclude_rule 'MD041'
# Code block style
exclude_rule 'MD046'
# File should end with a single newline character
exclude_rule 'MD047'
